/*
 * Copyright (c) 1993-2003, Silicon Graphics, Inc.
 * All Rights Reserved
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose and without fee is hereby granted, provided that the above
 * copyright notice appear in all copies and that both the copyright
 * notice and this permission notice appear in supporting documentation,
 * and that the name of Silicon Graphics, Inc. not be used in
 * advertising or publicity pertaining to distribution of the software
 * without specific, written prior permission.
 *
 * THE MATERIAL EMBODIED ON THIS SOFTWARE IS PROVIDED TO YOU "AS-IS" AND
 * WITHOUT WARRANTY OF ANY KIND, EXPRESS, IMPLIED OR OTHERWISE,
 * INCLUDING WITHOUT LIMITATION, ANY WARRANTY OF MERCHANTABILITY OR
 * FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT SHALL SILICON
 * GRAPHICS, INC.  BE LIABLE TO YOU OR ANYONE ELSE FOR ANY DIRECT,
 * SPECIAL, INCIDENTAL, INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND,
 * OR ANY DAMAGES WHATSOEVER, INCLUDING WITHOUT LIMITATION, LOSS OF
 * PROFIT, LOSS OF USE, SAVINGS OR REVENUE, OR THE CLAIMS OF THIRD
 * PARTIES, WHETHER OR NOT SILICON GRAPHICS, INC.  HAS BEEN ADVISED OF
 * THE POSSIBILITY OF SUCH LOSS, HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE POSSESSION, USE
 * OR PERFORMANCE OF THIS SOFTWARE.
 *
 * US Government Users Restricted Rights 
 * Use, duplication, or disclosure by the Government is subject to
 * restrictions set forth in FAR 52.227.19(c)(2) or subparagraph
 * (c)(1)(ii) of the Rights in Technical Data and Computer Software
 * clause at DFARS 252.227-7013 and/or in similar or successor clauses
 * in the FAR or the DOD or NASA FAR Supplement.  Unpublished - rights
 * reserved under the copyright laws of the United States.
 *
 * Contractor/manufacturer is:
 *	Silicon Graphics, Inc.
 *	1500 Crittenden Lane
 *	Mountain View, CA  94043
 *	United State of America
 *
 * OpenGL(R) is a registered trademark of Silicon Graphics, Inc.
 */

/*
 *  blendeqn.c
 *  Demonstrate the different blending functions available with the
 *  OpenGL imaging subset.  This program demonstrates use of the
 *  glBlendEquation() call.
 *
 *  The following keys change the selected blend equation function:
 *
 *      'a'  ->  GL_FUNC_ADD
 *      's'  ->  GL_FUNC_SUBTRACT
 *      'r'  ->  GL_FUNC_REVERSE_SUBTRACT
 *      'm'  ->  GL_MIN
 *      'x'  ->  GL_MAX
 */

#include <GL/glut.h>
#include <stdlib.h>


void init(void)
{
   glClearColor(1.0, 1.0, 0.0, 0.0);

   glBlendFunc(GL_ONE, GL_ONE);
   glEnable(GL_BLEND);
}

void display(void)
{
   glClear(GL_COLOR_BUFFER_BIT);

   glColor3f(0.0, 0.0, 1.0);
   glRectf(-0.5,-0.5,0.5,0.5);

   glFlush();
}

void reshape(int w, int h)
{
   GLdouble aspect = (GLdouble) w / h;

   glViewport(0, 0, (GLsizei) w, (GLsizei) h);
   glMatrixMode(GL_PROJECTION);
   glLoadIdentity();
   if (aspect < 1.0) {
     aspect = 1.0 / aspect;
     glOrtho(-aspect, aspect, -1.0, 1.0, -1.0, 1.0);
   } else
     glOrtho(-1.0, 1.0, -aspect, aspect, -1.0, 1.0);
   glMatrixMode(GL_MODELVIEW);
}

void keyboard(unsigned char key, int x, int y)
{
   switch (key) {
      case 'a': case 'A':
	 /* Colors are added as: (1, 1, 0) + (0, 0, 1) = (1, 1, 1)
	  *  which will produce a white square on a yellow background.
	  */
	 glBlendEquation(GL_FUNC_ADD);
	 break;

      case 's': case 'S':
	 /* Colors are subtracted as: (0, 0, 1) - (1, 1, 0) = (-1, -1, 1)
	  *  which is clamped to (0, 0, 1), producing a blue square on a
	  *  yellow background
	  */	 glBlendEquation(GL_FUNC_SUBTRACT);
	 break;

      case 'r': case 'R':
	 /* Colors are subtracted as: (1, 1, 0) - (0, 0, 1) = (1, 1, -1)
	  *  which is clamed to (1, 1, 0).  This produces yellow for both
	  *  the square and the background.
	  */
	 glBlendEquation(GL_FUNC_REVERSE_SUBTRACT);
	 break;

      case 'm': case 'M':
	 /* The minimum of each component is computed, as
	  *  [min(1, 0), min(1, 0), min(0, 1)] which equates to (0, 0, 0).
	  *  This will produce a black square on the yellow background.
	  */
	 glBlendEquation(GL_MIN);
	 break;

      case 'x': case 'X':
	 /* The minimum of each component is computed, as
	  *  [max(1, 0), max(1, 0), max(0, 1)] which equates to (1, 1, 1)
	  *  This will produce a white square on the yellow background.
	  */
	 glBlendEquation(GL_MAX);
	 break;

      case 27:
         exit(0);
   }
   
   glutPostRedisplay();
}

/*  Main Loop
 *  Open window with initial window size, title bar, 
 *  RGBA display mode, and handle input events.
 */
int main(int argc, char** argv)
{
   glutInit(&argc, argv);
   glutInitDisplayMode(GLUT_SINGLE | GLUT_RGB);
   glutInitWindowSize(512,512);
   glutInitWindowPosition(100, 100);
   glutCreateWindow(argv[0]);
   init();
   glutReshapeFunc(reshape);
   glutKeyboardFunc(keyboard);
   glutDisplayFunc(display);
   glutMainLoop();
   return 0;
}
